﻿using Microsoft.AspNetCore.Mvc;
using Microsoft.EntityFrameworkCore;
using System.Collections.Generic;
using System.Linq;
using System.Threading.Tasks;

using TestProject.Data;
using TestProject.Models;

namespace TestProject.Controllers
{
    /// <summary>
    /// 할일 컨트롤러
    /// </summary>
    [ApiController]
    [Route("api/[controller]")]
    public class TodoController : ControllerBase
    {
        //////////////////////////////////////////////////////////////////////////////////////////////////// Field
        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region Field

        /// <summary>
        /// 데이터베이스 컨텍스트
        /// </summary>
        private readonly DatabaseContext context;

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Constructor
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region 생성자 - TodoController(context)

        /// <summary>
        /// 생성자
        /// </summary>
        /// <param name="context">컨텍스트</param>
        public TodoController(DatabaseContext context)
        {
            this.context = context;
        }

        #endregion

        //////////////////////////////////////////////////////////////////////////////////////////////////// Method
        ////////////////////////////////////////////////////////////////////////////////////////// Public

        #region GET 처리하기 - Get()

        /// <summary>
        /// GET 처리하기
        /// </summary>
        /// <returns>액션 결과 태스크</returns>
        [HttpGet]
        public async Task<ActionResult<IEnumerable<TodoModel>>> Get()
        {
            return await this.context.Todo.ToListAsync();
        }

        #endregion
        #region GET 처리하기 - Get(id)

        /// <summary>
        /// GET 처리하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        [HttpGet("{id}")]
        public async Task<ActionResult<TodoModel>> Get(long id)
        {
            TodoModel todo = await this.context.Todo.FindAsync(id);

            if(todo == null)
            {
                return NotFound();
            }

            return todo;
        }

        #endregion
        #region POST 처리하기 - Post(todo)

        /// <summary>
        /// POST 처리하기
        /// </summary>
        /// <param name="todo">할일</param>
        /// <returns>액션 결과 태스크</returns>
        [HttpPost]
        public async Task<ActionResult<TodoModel>> Post(TodoModel todo)
        {
            this.context.Todo.Add(todo);

            await this.context.SaveChangesAsync();

            return CreatedAtAction("Get", new { id = todo.ID }, todo);
        }

        #endregion
        #region PUT 처리하기 - Put(id, todo)

        /// <summary>
        /// PUT 처리하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <param name="todo">할일</param>
        /// <returns>액션 결과 태스크</returns>
        [HttpPut("{id}")]
        public async Task<IActionResult> Put(long id, TodoModel todo)
        {
            if(id != todo.ID)
            {
                return BadRequest();
            }

            this.context.Entry(todo).State = EntityState.Modified;

            try
            {
                await this.context.SaveChangesAsync();
            }
            catch(DbUpdateConcurrencyException)
            {
                if(!TodoExists(id))
                {
                    return NotFound();
                }
                else
                {
                    throw;
                }
            }

            return NoContent();
        }

        #endregion
        #region DELETE 처리하기 - Delete(id)

        /// <summary>
        /// DELETE 처리하기
        /// </summary>
        /// <param name="id">ID</param>
        /// <returns>액션 결과 태스크</returns>
        [HttpDelete("{id}")]
        public async Task<ActionResult<TodoModel>> Delete(long id)
        {
            TodoModel todo = await this.context.Todo.FindAsync(id);

            if(todo == null)
            {
                return NotFound();
            }

            this.context.Todo.Remove(todo);

            await this.context.SaveChangesAsync();

            return todo;
        }

        #endregion

        ////////////////////////////////////////////////////////////////////////////////////////// Private

        #region 할일 존재 여부 구하기 - TodoExists(id)

        /// <summary>
        /// 할일 존재 여부 구하기
        /// </summary>
        /// <param name="id"></param>
        /// <returns>할일 존재 여부</returns>
        private bool TodoExists(long id)
        {
            return this.context.Todo.Any(e => e.ID == id);
        }

        #endregion
    }
}